"""
Entity module

This file is autogenerated from api specifications
"""

# Python 2 compatibility
from __future__ import unicode_literals
from builtins import str  # noqa
from builtins import super

from mbed_cloud.sdk.common.entity import Entity
from mbed_cloud.sdk.common import fields
from mbed_cloud.sdk import enums


class Device(Entity):
    """Represents the `Device` entity in Mbed Cloud"""

    # all fields available on this entity
    _fieldnames = [
        "account_id",
        "auto_update",
        "bootstrap_expiration_date",
        "bootstrapped_timestamp",
        "ca_id",
        "connector_expiration_date",
        "created_at",
        "custom_attributes",
        "deployed_state",
        "deployment",
        "description",
        "device_class",
        "device_execution_mode",
        "device_key",
        "endpoint_name",
        "endpoint_type",
        "enrolment_list_timestamp",
        "firmware_checksum",
        "host_gateway",
        "id",
        "manifest",
        "manifest_timestamp",
        "mechanism",
        "mechanism_url",
        "name",
        "serial_number",
        "state",
        "updated_at",
        "vendor_id",
    ]

    # common renames used when mapping {<API spec>: <SDK>}
    _renames = {}

    def __init__(
        self,
        _client=None,
        account_id=None,
        auto_update=None,
        bootstrap_expiration_date=None,
        bootstrapped_timestamp=None,
        ca_id=None,
        connector_expiration_date=None,
        created_at=None,
        custom_attributes=None,
        deployed_state=None,
        deployment=None,
        description=None,
        device_class=None,
        device_execution_mode=None,
        device_key=None,
        endpoint_name=None,
        endpoint_type=None,
        enrolment_list_timestamp=None,
        firmware_checksum=None,
        host_gateway=None,
        id=None,
        manifest=None,
        manifest_timestamp=None,
        mechanism=None,
        mechanism_url=None,
        name=None,
        serial_number=None,
        state=None,
        updated_at=None,
        vendor_id=None,
    ):
        """Creates a local `Device` instance

        :param account_id: The ID of the associated account.
        :type account_id: str
        :param auto_update: DEPRECATED: Mark this device for automatic firmware update.
        :type auto_update: bool
        :param bootstrap_expiration_date: The expiration date of the certificate used to connect to
            bootstrap server.
        :type bootstrap_expiration_date: date
        :param bootstrapped_timestamp: The timestamp of the device's most recent bootstrap process.
        :type bootstrapped_timestamp: datetime
        :param ca_id: The certificate issuer's ID.
        :type ca_id: str
        :param connector_expiration_date: The expiration date of the certificate used to connect to LwM2M
            server.
        :type connector_expiration_date: date
        :param created_at: The timestamp of when the device was created in the device
            directory.
        :type created_at: datetime
        :param custom_attributes: Up to five custom key-value attributes.
        :type custom_attributes: dict
        :param deployed_state: DEPRECATED: The state of the device's deployment.
        :type deployed_state: str
        :param deployment: DEPRECATED: The last deployment used on the device.
        :type deployment: str
        :param description: The description of the device.
        :type description: str
        :param device_class: An ID representing the model and hardware revision of the device.
        :type device_class: str
        :param device_execution_mode: The execution mode from the certificate of the device. Defaults to
            inheriting from host_gateway device.
            Permitted values:
              - 0 -
            unspecified execution mode (default if host_gateway invalid or not
            set)
              - 1 - development devices
              - 5 - production devices
        :type device_execution_mode: int
        :param device_key: The fingerprint of the device certificate.
        :type device_key: str
        :param endpoint_name: The endpoint name given to the device.
        :type endpoint_name: str
        :param endpoint_type: The endpoint type of the device. For example, the device is a
            gateway.
        :type endpoint_type: str
        :param enrolment_list_timestamp: The claim date/time.
        :type enrolment_list_timestamp: datetime
        :param firmware_checksum: The SHA256 checksum of the current firmware image.
        :type firmware_checksum: str
        :param host_gateway: The `endpoint_name` of the host gateway, if appropriate.
        :type host_gateway: str
        :param id: The ID of the device. The device ID is used across all Device
            Management APIs.
        :type id: str
        :param manifest: DEPRECATED: The URL for the current device manifest.
        :type manifest: str
        :param manifest_timestamp: The timestamp of the current manifest version.
        :type manifest_timestamp: datetime
        :param mechanism: The ID of the channel used to communicate with the device.
        :type mechanism: str
        :param mechanism_url: The address of the connector to use.
        :type mechanism_url: str
        :param name: The name of the device.
        :type name: str
        :param serial_number: The serial number of the device.
        :type serial_number: str
        :param state: The current state of the device.
        :type state: str
        :param updated_at: The time the object was updated.
        :type updated_at: datetime
        :param vendor_id: The device vendor ID.
        :type vendor_id: str
        """

        super().__init__(_client=_client)

        # inline imports for avoiding circular references and bulk imports

        # fields
        self._account_id = fields.StringField(value=account_id)
        self._auto_update = fields.BooleanField(value=auto_update)
        self._bootstrap_expiration_date = fields.DateField(
            value=bootstrap_expiration_date
        )
        self._bootstrapped_timestamp = fields.DateTimeField(
            value=bootstrapped_timestamp
        )
        self._ca_id = fields.StringField(value=ca_id)
        self._connector_expiration_date = fields.DateField(
            value=connector_expiration_date
        )
        self._created_at = fields.DateTimeField(value=created_at)
        self._custom_attributes = fields.DictField(value=custom_attributes)
        self._deployed_state = fields.StringField(
            value=deployed_state, enum=enums.DeviceDeployedStateEnum
        )
        self._deployment = fields.StringField(value=deployment)
        self._description = fields.StringField(value=description)
        self._device_class = fields.StringField(value=device_class)
        self._device_execution_mode = fields.IntegerField(value=device_execution_mode)
        self._device_key = fields.StringField(value=device_key)
        self._endpoint_name = fields.StringField(value=endpoint_name)
        self._endpoint_type = fields.StringField(value=endpoint_type)
        self._enrolment_list_timestamp = fields.DateTimeField(
            value=enrolment_list_timestamp
        )
        self._firmware_checksum = fields.StringField(value=firmware_checksum)
        self._host_gateway = fields.StringField(value=host_gateway)
        self._id = fields.StringField(value=id)
        self._manifest = fields.StringField(value=manifest)
        self._manifest_timestamp = fields.DateTimeField(value=manifest_timestamp)
        self._mechanism = fields.StringField(
            value=mechanism, enum=enums.DeviceMechanismEnum
        )
        self._mechanism_url = fields.StringField(value=mechanism_url)
        self._name = fields.StringField(value=name)
        self._serial_number = fields.StringField(value=serial_number)
        self._state = fields.StringField(value=state, enum=enums.DeviceStateEnum)
        self._updated_at = fields.DateTimeField(value=updated_at)
        self._vendor_id = fields.StringField(value=vendor_id)

    @property
    def account_id(self):
        """The ID of the associated account.
        
        api example: '00000000000000000000000000000000'
        
        :rtype: str
        """

        return self._account_id.value

    @account_id.setter
    def account_id(self, value):
        """Set value of `account_id`

        :param value: value to set
        :type value: str
        """

        self._account_id.set(value)

    @property
    def auto_update(self):
        """DEPRECATED: Mark this device for automatic firmware update.
        
        :rtype: bool
        """

        return self._auto_update.value

    @auto_update.setter
    def auto_update(self, value):
        """Set value of `auto_update`

        :param value: value to set
        :type value: bool
        """

        self._auto_update.set(value)

    @property
    def bootstrap_expiration_date(self):
        """The expiration date of the certificate used to connect to bootstrap server.
        
        :rtype: date
        """

        return self._bootstrap_expiration_date.value

    @bootstrap_expiration_date.setter
    def bootstrap_expiration_date(self, value):
        """Set value of `bootstrap_expiration_date`

        :param value: value to set
        :type value: date
        """

        self._bootstrap_expiration_date.set(value)

    @property
    def bootstrapped_timestamp(self):
        """The timestamp of the device's most recent bootstrap process.
        
        api example: '2017-05-22T12:37:55.576563Z'
        
        :rtype: datetime
        """

        return self._bootstrapped_timestamp.value

    @bootstrapped_timestamp.setter
    def bootstrapped_timestamp(self, value):
        """Set value of `bootstrapped_timestamp`

        :param value: value to set
        :type value: datetime
        """

        self._bootstrapped_timestamp.set(value)

    @property
    def ca_id(self):
        """The certificate issuer's ID.
        
        api example: '00000000000000000000000000000000'
        
        :rtype: str
        """

        return self._ca_id.value

    @ca_id.setter
    def ca_id(self, value):
        """Set value of `ca_id`

        :param value: value to set
        :type value: str
        """

        self._ca_id.set(value)

    @property
    def connector_expiration_date(self):
        """The expiration date of the certificate used to connect to LwM2M server.
        
        :rtype: date
        """

        return self._connector_expiration_date.value

    @connector_expiration_date.setter
    def connector_expiration_date(self, value):
        """Set value of `connector_expiration_date`

        :param value: value to set
        :type value: date
        """

        self._connector_expiration_date.set(value)

    @property
    def created_at(self):
        """The timestamp of when the device was created in the device directory.
        
        api example: '2017-05-22T12:37:55.576563Z'
        
        :rtype: datetime
        """

        return self._created_at.value

    @created_at.setter
    def created_at(self, value):
        """Set value of `created_at`

        :param value: value to set
        :type value: datetime
        """

        self._created_at.set(value)

    @property
    def custom_attributes(self):
        """Up to five custom key-value attributes.
        
        api example: "{ 'key': 'value' }"
        
        :rtype: dict
        """

        return self._custom_attributes.value

    @custom_attributes.setter
    def custom_attributes(self, value):
        """Set value of `custom_attributes`

        :param value: value to set
        :type value: dict
        """

        self._custom_attributes.set(value)

    @property
    def deployed_state(self):
        """DEPRECATED: The state of the device's deployment.
        
        :rtype: str
        """

        return self._deployed_state.value

    @deployed_state.setter
    def deployed_state(self, value):
        """Set value of `deployed_state`

        :param value: value to set
        :type value: str
        """

        self._deployed_state.set(value)

    @property
    def deployment(self):
        """DEPRECATED: The last deployment used on the device.
        
        :rtype: str
        """

        return self._deployment.value

    @deployment.setter
    def deployment(self, value):
        """Set value of `deployment`

        :param value: value to set
        :type value: str
        """

        self._deployment.set(value)

    @property
    def description(self):
        """The description of the device.
        
        api example: 'description'
        
        :rtype: str
        """

        return self._description.value

    @description.setter
    def description(self, value):
        """Set value of `description`

        :param value: value to set
        :type value: str
        """

        self._description.set(value)

    @property
    def device_class(self):
        """An ID representing the model and hardware revision of the device.
        
        :rtype: str
        """

        return self._device_class.value

    @device_class.setter
    def device_class(self, value):
        """Set value of `device_class`

        :param value: value to set
        :type value: str
        """

        self._device_class.set(value)

    @property
    def device_execution_mode(self):
        """The execution mode from the certificate of the device. Defaults to inheriting
        from host_gateway device.
        Permitted values:
          - 0 - unspecified execution mode
        (default if host_gateway invalid or not set)
          - 1 - development devices
          - 5
        - production devices
        
        :rtype: int
        """

        return self._device_execution_mode.value

    @device_execution_mode.setter
    def device_execution_mode(self, value):
        """Set value of `device_execution_mode`

        :param value: value to set
        :type value: int
        """

        self._device_execution_mode.set(value)

    @property
    def device_key(self):
        """The fingerprint of the device certificate.
        
        api example: '00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00:00
            :00:00:00:00:00:00'
        
        :rtype: str
        """

        return self._device_key.value

    @device_key.setter
    def device_key(self, value):
        """Set value of `device_key`

        :param value: value to set
        :type value: str
        """

        self._device_key.set(value)

    @property
    def endpoint_name(self):
        """The endpoint name given to the device.
        
        api example: '00000000-0000-0000-0000-000000000000'
        
        :rtype: str
        """

        return self._endpoint_name.value

    @endpoint_name.setter
    def endpoint_name(self, value):
        """Set value of `endpoint_name`

        :param value: value to set
        :type value: str
        """

        self._endpoint_name.set(value)

    @property
    def endpoint_type(self):
        """The endpoint type of the device. For example, the device is a gateway.
        
        :rtype: str
        """

        return self._endpoint_type.value

    @endpoint_type.setter
    def endpoint_type(self, value):
        """Set value of `endpoint_type`

        :param value: value to set
        :type value: str
        """

        self._endpoint_type.set(value)

    @property
    def enrolment_list_timestamp(self):
        """The claim date/time.
        
        api example: '2017-05-22T12:37:55.576563Z'
        
        :rtype: datetime
        """

        return self._enrolment_list_timestamp.value

    @enrolment_list_timestamp.setter
    def enrolment_list_timestamp(self, value):
        """Set value of `enrolment_list_timestamp`

        :param value: value to set
        :type value: datetime
        """

        self._enrolment_list_timestamp.set(value)

    @property
    def firmware_checksum(self):
        """The SHA256 checksum of the current firmware image.
        
        api example: '0000000000000000000000000000000000000000000000000000000000000000'
        
        :rtype: str
        """

        return self._firmware_checksum.value

    @firmware_checksum.setter
    def firmware_checksum(self, value):
        """Set value of `firmware_checksum`

        :param value: value to set
        :type value: str
        """

        self._firmware_checksum.set(value)

    @property
    def host_gateway(self):
        """The `endpoint_name` of the host gateway, if appropriate.
        
        :rtype: str
        """

        return self._host_gateway.value

    @host_gateway.setter
    def host_gateway(self, value):
        """Set value of `host_gateway`

        :param value: value to set
        :type value: str
        """

        self._host_gateway.set(value)

    @property
    def id(self):
        """The ID of the device. The device ID is used across all Device Management APIs.
        
        api example: '00000000000000000000000000000000'
        
        :rtype: str
        """

        return self._id.value

    @id.setter
    def id(self, value):
        """Set value of `id`

        :param value: value to set
        :type value: str
        """

        self._id.set(value)

    @property
    def manifest(self):
        """DEPRECATED: The URL for the current device manifest.
        
        :rtype: str
        """

        return self._manifest.value

    @manifest.setter
    def manifest(self, value):
        """Set value of `manifest`

        :param value: value to set
        :type value: str
        """

        self._manifest.set(value)

    @property
    def manifest_timestamp(self):
        """The timestamp of the current manifest version.
        
        api example: '2017-05-22T12:37:55.576563Z'
        
        :rtype: datetime
        """

        return self._manifest_timestamp.value

    @manifest_timestamp.setter
    def manifest_timestamp(self, value):
        """Set value of `manifest_timestamp`

        :param value: value to set
        :type value: datetime
        """

        self._manifest_timestamp.set(value)

    @property
    def mechanism(self):
        """The ID of the channel used to communicate with the device.
        
        :rtype: str
        """

        return self._mechanism.value

    @mechanism.setter
    def mechanism(self, value):
        """Set value of `mechanism`

        :param value: value to set
        :type value: str
        """

        self._mechanism.set(value)

    @property
    def mechanism_url(self):
        """The address of the connector to use.
        
        :rtype: str
        """

        return self._mechanism_url.value

    @mechanism_url.setter
    def mechanism_url(self, value):
        """Set value of `mechanism_url`

        :param value: value to set
        :type value: str
        """

        self._mechanism_url.set(value)

    @property
    def name(self):
        """The name of the device.
        
        api example: '00000000-0000-0000-0000-000000000000'
        
        :rtype: str
        """

        return self._name.value

    @name.setter
    def name(self, value):
        """Set value of `name`

        :param value: value to set
        :type value: str
        """

        self._name.set(value)

    @property
    def serial_number(self):
        """The serial number of the device.
        
        api example: '00000000-0000-0000-0000-000000000000'
        
        :rtype: str
        """

        return self._serial_number.value

    @serial_number.setter
    def serial_number(self, value):
        """Set value of `serial_number`

        :param value: value to set
        :type value: str
        """

        self._serial_number.set(value)

    @property
    def state(self):
        """The current state of the device.
        
        :rtype: str
        """

        return self._state.value

    @state.setter
    def state(self, value):
        """Set value of `state`

        :param value: value to set
        :type value: str
        """

        self._state.set(value)

    @property
    def updated_at(self):
        """The time the object was updated.
        
        api example: '2017-05-22T12:37:55.576563Z'
        
        :rtype: datetime
        """

        return self._updated_at.value

    @updated_at.setter
    def updated_at(self, value):
        """Set value of `updated_at`

        :param value: value to set
        :type value: datetime
        """

        self._updated_at.set(value)

    @property
    def vendor_id(self):
        """The device vendor ID.
        
        api example: '00000000-0000-0000-0000-000000000000'
        
        :rtype: str
        """

        return self._vendor_id.value

    @vendor_id.setter
    def vendor_id(self, value):
        """Set value of `vendor_id`

        :param value: value to set
        :type value: str
        """

        self._vendor_id.set(value)

    def create(self):
        """Create a device

        api documentation:
        https://os.mbed.com/search/?q=service+apis+/v3/devices/
        
        :rtype: Device
        """

        return self._client.call_api(
            method="post",
            path="/v3/devices/",
            body_params={
                "auto_update": self._auto_update.to_api(),
                "bootstrap_expiration_date": self._bootstrap_expiration_date.to_api(),
                "bootstrapped_timestamp": self._bootstrapped_timestamp.to_api(),
                "ca_id": self._ca_id.to_api(),
                "connector_expiration_date": self._connector_expiration_date.to_api(),
                "custom_attributes": self._custom_attributes.to_api(),
                "deployment": self._deployment.to_api(),
                "description": self._description.to_api(),
                "device_class": self._device_class.to_api(),
                "device_execution_mode": self._device_execution_mode.to_api(),
                "device_key": self._device_key.to_api(),
                "endpoint_name": self._endpoint_name.to_api(),
                "endpoint_type": self._endpoint_type.to_api(),
                "firmware_checksum": self._firmware_checksum.to_api(),
                "host_gateway": self._host_gateway.to_api(),
                "manifest": self._manifest.to_api(),
                "mechanism": self._mechanism.to_api(),
                "mechanism_url": self._mechanism_url.to_api(),
                "name": self._name.to_api(),
                "serial_number": self._serial_number.to_api(),
                "state": self._state.to_api(),
                "vendor_id": self._vendor_id.to_api(),
            },
            unpack=self,
        )

    def delete(self):
        """Delete a device.

        api documentation:
        https://os.mbed.com/search/?q=service+apis+/v3/devices/{id}/
        
        :rtype: Device
        """

        return self._client.call_api(
            method="delete",
            path="/v3/devices/{id}/",
            path_params={"id": self._id.to_api()},
            unpack=self,
        )

    def get(self):
        """Get a devices

        api documentation:
        https://os.mbed.com/search/?q=service+apis+/v3/devices/{id}/
        
        :rtype: Device
        """

        return self._client.call_api(
            method="get",
            path="/v3/devices/{id}/",
            path_params={"id": self._id.to_api()},
            unpack=self,
        )

    def list(self, include=None, max_results=None, page_size=None, order=None):
        """List all devices.

        api documentation:
        https://os.mbed.com/search/?q=service+apis+/v3/devices/
        
        :param include: Comma-separated list of data fields to return. Currently supported:
            `total_count`.
        :type include: str
        
        :param max_results: Total maximum number of results to retrieve
        :type max_results: int
            
        :param page_size: How many objects to retrieve in the page.
        :type page_size: int
        
        :param order: The order of the records based on creation time, `ASC` or `DESC`; by
            default `ASC`.
        :type order: str
        
        :return: An iterator object which yields instances of an entity.
        :rtype: mbed_cloud.pagination.PaginatedResponse(Device)
        """

        from mbed_cloud.sdk.common._custom_methods import paginate
        from mbed_cloud.sdk.entities import Device

        return paginate(
            self=self,
            foreign_key=Device,
            include=include,
            max_results=max_results,
            page_size=page_size,
            order=order,
            wraps=self._paginate_list,
        )

    def _paginate_list(self, after=None, include=None, limit=None, order=None):
        """List all devices.

        api documentation:
        https://os.mbed.com/search/?q=service+apis+/v3/devices/
        
        :param after: The ID of The item after which to retrieve the next page.
        :type after: str
        
        :param include: Comma-separated list of data fields to return. Currently supported:
            `total_count`.
        :type include: str
        
        :param limit: How many objects to retrieve in the page.
        :type limit: int
        
        :param order: The order of the records based on creation time, `ASC` or `DESC`; by
            default `ASC`.
        :type order: str
        
        :rtype: mbed_cloud.pagination.PaginatedResponse
        """

        return self._client.call_api(
            method="get",
            path="/v3/devices/",
            query_params={
                "after": fields.StringField(after).to_api(),
                "include": fields.StringField(include).to_api(),
                "limit": fields.IntegerField(limit).to_api(),
                "order": fields.StringField(order).to_api(),
            },
            unpack=False,
        )

    def renew_certificate(self, certificate_name):
        """Request certificate renewal.

        api documentation:
        https://os.mbed.com/search/?q=service+apis+/v3/devices/{device-id}/certificates/{certificate-name}/renew
        
        :param certificate_name: The certificate name.
        :type certificate_name: str
        
        :rtype: CertificateEnrollment
        """

        from mbed_cloud.sdk.entities import CertificateEnrollment

        return self._client.call_api(
            method="post",
            path="/v3/devices/{device-id}/certificates/{certificate-name}/renew",
            path_params={
                "certificate-name": fields.StringField(certificate_name).to_api(),
                "device-id": self._id.to_api(),
            },
            unpack=CertificateEnrollment,
        )

    def update(self):
        """Update a device

        api documentation:
        https://os.mbed.com/search/?q=service+apis+/v3/devices/{id}/
        
        :rtype: Device
        """

        return self._client.call_api(
            method="put",
            path="/v3/devices/{id}/",
            body_params={
                "auto_update": self._auto_update.to_api(),
                "ca_id": self._ca_id.to_api(),
                "custom_attributes": self._custom_attributes.to_api(),
                "description": self._description.to_api(),
                "device_key": self._device_key.to_api(),
                "endpoint_name": self._endpoint_name.to_api(),
                "endpoint_type": self._endpoint_type.to_api(),
                "host_gateway": self._host_gateway.to_api(),
                "name": self._name.to_api(),
            },
            path_params={"id": self._id.to_api()},
            unpack=self,
        )
